@** Debug code.

@ Headers needed for debug code.
@c
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "types.h"

#ifdef DEBUG
#include "protocol.h"
#endif

@* Exported interfaces.

These macros and functions are exported via the {\tt debug.h} file and used in 
most of the other files.

@(debug.h@>=
#ifndef _CCID_DEBUG_H
#define _CCID_DEBUG_H

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "types.h"

#ifdef DEBUG
#include "protocol.h"
#endif

#ifdef DEBUG
#define debug(a, b...) do { fprintf(stderr, "[Debug] "); \
                            fprintf(stderr, a, ##b); \
                            fprintf(stderr, "\n"); } while(0)
#else
#define debug(...)
#endif

#ifdef DEBUG
#define print_ccid_request(a) real_print_ccid_request(a)
void real_print_ccid_request(ccid_request* req);
#else
#define print_ccid_request(a)
#endif

#ifdef DEBUG
#define print_ccid_response(a) real_print_ccid_response(a)
void real_print_ccid_response(ccid_response* res);
#else
#define print_ccid_response(a)
#endif

#endif

@* Procedure |real_print_ccid_request()|.
@c
#ifdef DEBUG
void real_print_ccid_request(ccid_request* req)
{
   fprintf(stderr, "request {\n\tprotocol: %d\n\tdsize: %d\n\tapdu {\n\
                    \t\ttype: %d\n",
                   req->protocol,
                   req->dsize,
                   req->apdu.apdu_type);
   switch (req->apdu.apdu_type) {
      case APDU_AUTHENTICATE:
         fprintf(stderr, "\t\tauth {\n\t\t\tinternal: %s\n\t\t\tglobalsecret: \
                          %s\n\t\t\talgorithmid: %d\n\t\t\tsecretid: %d\n\
                          \t\t\tresponselength: %d\n\t\t}\n",
                         req->apdu.auth.internal?"true":"false",
                         req->apdu.auth.globalsecret?"true":"false",
                         req->apdu.auth.algorithmid,
                         req->apdu.auth.secretid,
                         req->apdu.auth.responselength);
         break;
      case APDU_BINARY:
         fprintf(stderr, "\t\tbinary {\n\t\t\tmode: %d\n\t\t\toffset: %d\
                          \n\t\t\tlength: %d\n\t\t}\n",
                         req->apdu.binary.mode,
                         req->apdu.binary.offset,
                         req->apdu.binary.length);
         break;
      case APDU_CHALLENGE:
         fprintf(stderr, "\t\tchallenge {\n\t\t\tmaxlength: %d\n\t\t}\n",
                         req->apdu.challenge.maxlength);
         break;
      case APDU_CHANNEL:
         fprintf(stderr, "\t\tchannel {\n\t\t\topen: %s\n\t\t\tchannelno: %d\
                          \n\t\t}\n",
                         req->apdu.channel.open?"true":"false",
                         req->apdu.channel.channelno);
         break;
      case APDU_DATA:
         fprintf(stderr, "\t\tdata {\n\t\t\tmode: %d\n\t\t\ttagtype: %d\
                          \n\t\t\ttag: %d\n\t\t\tlength: %d\n\t\t}\n",
                         req->apdu.data.mode,
                         req->apdu.data.tagtype,
                         req->apdu.data.tag,
                         req->apdu.data.length);
         break;
      case APDU_RECORD:
         fprintf(stderr, "\t\trecord {\n\t\t\tmode: %d\n\t\t\ttype: %d\
                          \n\t\t\trecord: %d\n\t\t\tlength: %d\n\t\t\t\
                          ef: %d\n\t\t}\n",
                         req->apdu.record.mode,
                         req->apdu.record.type,
                         req->apdu.record.record,
                         req->apdu.record.length,
                         req->apdu.record.ef);
         break;
      case APDU_SELECT:
         fprintf(stderr, "\t\tselect {\n\t\t\tmode: %d\n\t\t\trtemplate: %d\
                          \n\t\t\trsize: %d\n",
                         req->apdu.select.mode,
                         req->apdu.select.rtemplate,
                         req->apdu.select.rsize);
         switch (req->apdu.select.mode) {
            case DF:
               fprintf(stderr, "\t\t\ttype: %d\n\t\t\tDF: %d\n",
                               req->apdu.select.type,
                               req->apdu.select.DF);
               break;
            case PATH:
               fprintf(stderr, "\t\t\trelative: %s\n",
                               req->apdu.select.relative?"true":"false");
               break;
            case ID:
               break;
         }
         fprintf(stderr, "\t\t}\n");
         break;
      case APDU_VERIFY:
         fprintf(stderr, "\t\tverify {\n\t\t\tglobal: %s\n\t\t\treference: %d\
                          \n\t\t}\n",
                         req->apdu.verify.global?"true":"false",
                         req->apdu.verify.reference);
   }
   fprintf(stderr, "\t}\n}\n");
}
#endif

@* Procedure |real_print_ccid_response()|.
@c
#ifdef DEBUG
void real_print_ccid_response(ccid_response* res)
{
   fprintf(stderr, "response {\n\tprotocol: %d\n\trsize: %d\n\tapdu {\n\
\t\ttype: %d\n\t\tstatus: %d\n\t}\n}\n",
                   res->protocol,
                   res->rsize,
                   res->apdu.apdu_type,
                   res->apdu.status);

}
#endif
